class ViewBackend:
    def authenticate(self, *args):
        """
        Always return ``None`` to prevent authentication within this backend.
        """
        return None

    def has_perm(self, user_obj, perm, obj=None):  # noqa
        try:
            if not obj.authenticate:
                return True
        except AttributeError:
            return False

        if not user_obj.is_authenticated:
            return False

        if user_obj.is_superuser:
            return True

        try:
            if obj.allowed_groups == 'any':
                return True
        except AttributeError:
            return False

        for group in user_obj.groups.all():
            if group.name in obj.allowed_groups:
                return True

        return False



from django import http
from django.contrib import messages


class FormMixin:
    """Mixin for views which have a Form."""

    success_url_next = True
    initial = {}
    default_template_name = 'crudlfap/form.html'

    def get_context(self, **context):
        context['form'] = self.form
        return super().get_context(**context)

    def form_valid(self):
        """If the form is valid, redirect to the supplied URL."""
        self.message_success()
        return http.HttpResponseRedirect(self.success_url)

    def form_invalid(self):
        """If the form is invalid, render the invalid form."""
        self.message_error()
        return self.render_to_response()

    def get_title_submit(self):
        """
        Title of the submit button.

        Defaults to :py:attr:`~crudlfap.mixins.menu.MenuMixin.title_menu`
        """
        return self.title_menu

    def post(self, request, *args, **kwargs):
        """
        Handle POST requests: instantiate a form instance with the passed
        POST variables and then check if it's valid.
        """
        if self.form.is_valid():
            return self.form_valid()
        else:
            return self.form_invalid()

    def get_initial(self):
        """Return the initial data to use for forms on this view."""
        return self.initial.copy()

    def get_prefix(self):
        """Return the prefix to use for forms."""
        return None

    def get_form_class(self):
        if self.router:
            return getattr(self.router, 'form_class')

    def get_form(self):
        """Return an instance of the form to be used in this view."""
        self.form = self.form_class(**self.get_form_kwargs())
        return self.form

    def get_form_kwargs(self):
        """Return the keyword arguments for instantiating the form."""
        self.form_kwargs = {
            'initial': self.get_initial(),
            'prefix': self.get_prefix(),
        }

        if self.request.method in ('POST', 'PUT'):
            self.form_kwargs.update({
                'data': self.request.POST,
                'files': self.request.FILES,
            })
        return self.form_kwargs

    def get_next_url(self):
        if '_next' in self.request.POST:
            self.next_url = self.request.POST.get('_next')
        if '_next' in self.request.GET:
            self.next_url = self.request.GET.get('_next')

    def get_success_url(self):
        if self.next_url:
            return self.next_url

        if (hasattr(self, 'object')
                and hasattr(self.object, 'get_absolute_url')):
            return self.object.get_absolute_url()

        if self.router['list']:
            return self.router['list'].url

        return super().get_success_url()

    def message_html(self, message):
        return message

    def message_success(self):
        messages.success(
            self.request,
            self.message_html(self.form_valid_message)
        )

    def message_error(self):
        messages.error(
            self.request,
            self.message_html(self.form_invalid_message)
        )



import importlib
from typing import Any, Dict, List, Optional

SOURCEREF = List[Dict[str, Optional[Dict[str, Any]]]]

__all__ = (
    'add_optional_dep',
    'install_optional',
)


if 'ModuleNotFoundError' not in globals():
    ModuleNotFoundError = ImportError


def module_installed(module: str) -> bool:
    """
    Determines if a given module string can be resolved

    Determine if the module referenced by string, can be imported by trying
    an import in two ways:

    - direct import of the module
    - import of the module minus the last part, then see if the last part is
      an attribute of the module.

    Parts of course, are separated by dots.

    :param module: module reference
    :return: True if importable, False otherwise
    """
    have_module = False
    try:
        importlib.__import__(module)
    except ModuleNotFoundError:
        mod_path, dot, cls = module.rpartition('.')
        if not mod_path:
            return False
        try:
            mod = importlib.import_module(mod_path)
        except ModuleNotFoundError:
            return False
        else:
            if hasattr(mod, cls):
                have_module = True
    else:
        have_module = True

    return have_module


def add_optional_dep(module: str, to: List[str], before: str = None,
                     after: str = None):
    """
    Adds an optional dependency

    Add an optional dependency to the given `to` list, if it is resolvable
    by the importer.

    The module can be inserted at the right spot by using before or after
    keyword arguments. If both are given, the gun is pointing at your feet
    and before wins. If neither are given, the module is appended at the
    end.

    :param module: module to add, as it would be added to the given `to` list
    :param to: list to add the module to
    :param before: module string as should be available in the to list.
    :param after: module string as should be available in the to list.
    """
    if not module_installed(module):
        return

    if not before and not after:
        to.append(module)
        return

    try:
        if before:
            pos = to.index(before)
        else:
            pos = to.index(after) + 1
    except ValueError:
        pass
    else:
        to.insert(pos, module)


def install_optional(source: SOURCEREF, target: List[str]):
    """
    Install optional modules

    Add a module as provided by the source reference to the target list. The
    source reference is a list of dictionaries:

    - key: module reference that needs to be inserted
    - value: a dictionary matching the keyword arguments to `add_optional_dep`:
        - before: module reference in `target`; the module will be inserted
                  before this module.
        - after: module reference in `target`; the module will be inserted
                 after this module.
        If value is `None`, the module will be appended to `target`.

    :param source: modules to install
    :param target: install into this list.
    """
    for app in source:
        for ref, kwargs in app.items():
            kwargs = kwargs or {}
            add_optional_dep(ref, target, **kwargs)



from django.conf import settings
from django.urls import include, path, re_path

from crudlfap import shortcuts as crudlfap

urlpatterns = [
    crudlfap.site.urlpattern,
    path('auth/', include('django.contrib.auth.urls')),
    path('bundles/', include('ryzom_django.bundle')),
]

# CRUDLFA+ extras
if 'crudlfap_registration' in settings.INSTALLED_APPS:
    urlpatterns.append(
        path(
            'registration/',
            include('django_registration.backends.activation.urls')
        ),
    )

if 'debug_toolbar' in settings.INSTALLED_APPS and settings.DEBUG:
    import debug_toolbar
    urlpatterns += [
        re_path(r'^__debug__/', include(debug_toolbar.urls)),
    ]



# Generated by Django 3.1.7 on 2021-04-06 09:29

import django.contrib.sites.models
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('sites', '0002_alter_domain_unique'),
    ]

    operations = [
        migrations.CreateModel(
            name='Site',
            fields=[
                ('site_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='sites.site')),
                ('settings', models.JSONField(blank=True, default={})),
                ('port', models.PositiveIntegerField(null=True)),
                ('protocol', models.CharField(default='http', max_length=5)),
            ],
            bases=('sites.site',),
            managers=[
                ('objects', django.contrib.sites.models.SiteManager()),
            ],
        ),
    ]



"""CRUD :eMixins that can be used in Actions or Views."""
import copy

from django import forms
from django.contrib.admin.models import ADDITION, CHANGE, DELETION, LogEntry
from django.contrib.contenttypes.models import ContentType

from crudlfap import html


class CreateMixin:
    style = 'success'
    icon = 'add'
    default_template_name = 'crudlfap/create.html'
    template_name_suffixes = ['create', 'form']
    controller = 'modal'
    action = 'click->modal#open'
    color = 'green'
    log_action_flag = ADDITION
    menus = ['model']
    permission_shortcode = 'add'

    def form_valid(self):
        self.object = self.form.save()
        return super().form_valid()


class ActionMixin:
    def has_perm(self):
        """
        Call has_perm_object or return True.

        When called without an object, return True if router agrees.

        Otherwise, return the result of has_perm_object(), that you must
        implement, to return wether permission is accepted for a particular
        object.

        To override yourself, again go inside an if super().has_perm(): to
        benefit from this behaviour.
        """
        if super().has_perm():
            if hasattr(self, 'object'):
                return self.has_perm_object()
            return True

    def has_perm_object(self):
        """
        Override this method: test self.object then return True.

        By default, return True.
        """
        return True


class DeleteMixin(ActionMixin):
    style = 'danger'
    fa_icon = 'trash'
    icon = 'delete'
    success_url_next = True
    color = 'red'
    log_action_flag = DELETION
    controller = 'modal'
    action = 'click->modal#open'
    form_class = forms.Form
    permission_shortcode = 'delete'

    def form_valid(self):
        if hasattr(self, 'object_list'):
            self.result = copy.copy(self.object_list).delete()
        else:
            self.result = self.object.delete()
        return super().form_valid()

    def get_success_url(self):
        return self.router['list'].reverse()


class DetailMixin:
    fa_icon = 'search-plus'
    icon = 'search'
    default_template_name = 'crudlfap/detail.html'
    color = 'blue'
    menus_display = ['object', 'object_detail']

    @classmethod
    def get_urlpath(cls):
        """Identify the object by slug or pk in the pattern."""
        return r'<{}>'.format(cls.urlfield)

    def get_title(self):
        return str(self.object)

    def get_display_fields(self):
        self.display_fields = [
            {
                'field': self.model._meta.get_field(field),
                'value': self.get_field_display(field),
            }
            for field in (
                [f.name for f in self.model._meta.fields]
                if self.fields == '__all__'
                else self.fields
            ) if field not in self.exclude
        ]

    def get_field_display(self, name):
        value_getter = '_'.join(['get', name, 'display'])
        if hasattr(self.object, value_getter):
            return getattr(self.object, value_getter)()
        value = getattr(self.object, name)
        if hasattr(value, 'get_absolute_url'):
            return html.A(
                str(value),
                href=value.get_absolute_url(),
            ).render()
        return value


class HistoryMixin:
    icon = 'history'
    template_name_suffix = '_history'
    default_template_name = 'crudlfap/history.html'
    controller = None
    action = None

    def get_object_list(self):
        ctype = ContentType.objects.get_for_model(self.model)
        return LogEntry.objects.filter(
            content_type=ctype,
            object_id=self.object.pk,
        )


class ListMixin:
    default_template_name = 'crudlfap/list.html'
    icon = 'list'
    body_class = 'full-width'
    menus = ['main', 'model']
    title_heading = None

    def get_title(self):
        return self.model._meta.verbose_name_plural.capitalize()

    def get_urlpath(self):
        return ''

    def get_title_heading(self):
        return self.model._meta.verbose_name_plural.capitalize()


class UpdateMixin:
    icon = 'edit'
    default_template_name = 'crudlfap/update.html'
    template_name_suffixes = ['create', 'form']
    controller = 'modal'
    action = 'click->modal#open'
    color = 'orange'
    locks = True
    log_action_flag = CHANGE
    permission_shortcode = 'change'

    def get_form_fields(self):
        if hasattr(self, 'update_fields'):
            return self.update_fields
        if hasattr(self.router, 'update_fields'):
            return self.router.update_fields
        return super().get_form_fields()

    def form_valid(self):
        self.object = self.form.save()
        return super().form_valid()



from django.utils.translation import ugettext_lazy as _

from .registry import Registry
from .views.generic import TemplateView

site = Registry(
    views=[
        TemplateView.clone(
            icon='home',
            template_name='crudlfap/home.html',
            menus=['main'],
            title=_('Home'),
            title_menu=_('Home'),
            title_heading='',
            urlname='home',
            urlpath='',
            authenticate=False,
        )
    ]
)



# flake8: noqa: F401
"""Import everything we expose in crudlfap namespace."""

from .apps import _installed
from .factory import Factory
from .mixins import *  # noqa
from .registry import Registry
from .route import Route
from .router import Router, Views, ViewsDescriptor
from .site import site
from .views import *  # noqa



import os
import sys

from setuptools import find_packages, setup
from setuptools.command.install import install

setup(
    name='crudlfap',
    setup_requires='setupmeta',
    version='0.10.0',
    description='Rich frontend for generic views with Django',
    author='James Pic',
    author_email='jamespic@gmail.com',
    url='https://yourlabs.io/oss/crudlfap',
    packages=find_packages('src'),
    package_dir={'': 'src'},
    include_package_data=True,
    keywords='django crud',
    install_requires=[
        'ryzom>=0.6,<0.7',
        'django>=3.1,<3.2',
        'django-tables2',
        'django-filter',
        'timeago',
        'lookupy-unmanaged',
        'libsass',
    ],
    tests_require=['tox'],
    extras_require=dict(
        project=[
            'django-debug-toolbar',
            'django-extensions',
            'django-registration',
            'dj-static',
        ],
    ),
    classifiers=[
        'Development Status :: 1 - Planning',
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 3',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Software Development :: Libraries :: Python Modules',
    ],
    python_requires='>=3',
)



import collections

from django.apps import apps
from django.urls import path

from .factory import Factory
from .router import ViewsDescriptor

crudlfap = apps.get_app_config('crudlfap')  # pylint: disable=invalid-name


class Registry(Factory, collections.OrderedDict):
    views = ViewsDescriptor()

    def get_menu(self, name, request, **kwargs):
        result = []
        for view in self.views:
            menus = getattr(view, 'menus', [])
            if name in menus and view(request=request).has_perm():
                result.append(view)
        for model, router in self.items():
            menu = router.get_menu(name, request, **kwargs)
            if not menu:
                continue
            result += menu
        return result

    def get_app_menus(self, name, request, **kwargs):
        """Sort Router instances by app name."""
        result = collections.OrderedDict()
        for model, router in self.items():
            menu = router.get_menu(name, request, **kwargs)

            if not menu:
                continue

            app = apps.get_app_config(model._meta.app_label)
            result.setdefault(app, [])
            result[app].append(router)
        return result

    def __getitem__(self, arg):
        """Return a router instance by model class, instance or dotted name."""
        from django.db import models
        if isinstance(arg, models.Model):
            arg = type(arg)
        elif isinstance(arg, str) and '.' in arg:
            arg = apps.get_model(*arg.split('.'))
        return super().__getitem__(arg)

    def __init__(self, views=None, *a, **attrs):
        self.views = views or []
        super().__init__(*a)
        for k, v in attrs.items():
            setattr(self, k, v)

    def get_urlpatterns(self):
        for view in self.views:
            view.registry = self

        return [
            router.urlpattern for router in self.values()
        ] + [view.urlpattern for view in self.views]

    def get_urlpattern(self):
        urlpath = self.urlpath
        if urlpath and not urlpath.endswith('/'):
            urlpath += '/'

        return path(urlpath, (
            self.urlpatterns,
            self.app_name,
            self.namespace,
        ))

    def get_app_name(self):
        return 'crudlfap'

    def get_namespace(self):
        return None

    def get_urlpath(self):
        return ''

    def get_title(self):
        return 'CRUDLFA+'

    def get_navbar_color(self):
        return ''



from django.apps import apps
from django.urls import reverse
from crudlfap.html import *  # noqa

User = apps.get_model(getattr(settings, 'AUTH_USER_MODEL', 'auth.User'))


@template('django_registration/registration_form.html', App)
class RegistrationFormViewComponent(FormContainer):
    def to_html(self, *content, view, form, **context):
        return super().to_html(
            *content,
            H1('Looks like you need to register', cls='center-text'),
            Form(
                CSRFInput(view.request),
                form,
                MDCButtonRaised('Register'),
                method='POST',
                cls='form card',
            ),
        )


@template('django_registration/registration_complete.html', App)
class RegistrationCompleteNarrowCard(FormContainer):
    def __init__(self, **context):
        super().__init__(
            H4('Check your emails to finish !'),
            Div(
                'An activation link has been sent to your email address, '
                + 'please open it to finish the signup process.',
                style='margin-bottom: 24px',
            ),
            Div(
                'Then, come back and login to participate to your election.',
                style='margin-bottom: 24px;',
            ),
            cls='card',
            style='text-align: center',
        )


@template('django_registration/activation_complete.html', App)
class ActivationCompleteNarrowCard(FormContainer):
    def __init__(self, **context):
        super().__init__(
            H4('Your account has been activated !'),
            Div(
                'You may now ',
                A('login', href=reverse('login')),
                style='margin-bottom: 24px',
            ),
            cls='card',
            style='text-align: center',
        )


@template('django_registration/activation_failed.html', App)
class ActivationFailureNarrowCard(FormContainer):
    def __init__(self, **context):
        super().__init__(
            H4('Account activation failure'),
            Div(
                'Most likely your account has already been activated.',
                style='margin-bottom: 24px',
            ),
            cls='card',
            style='text-align: center',
        )



from crudlfap.settings import *  # noqa

INSTALLED_APPS += [  # noqa
    # CRUDLFA+ examples
    'crudlfap_example.artist',
    'crudlfap_example.song',
    'crudlfap_example.blog',

    # CRUDLFA+ extras
    'django_registration',
    'crudlfap_registration',
]

install_optional(OPTIONAL_APPS, INSTALLED_APPS)  # noqa
install_optional(OPTIONAL_MIDDLEWARE, MIDDLEWARE)  # noqa

AUTHENTICATION_BACKENDS += [  # noqa
    'crudlfap_example.blog.crudlfap.AuthBackend',
]
